/*      DS18B20 */
#include "common_defs.h"

UINT8 scratchpad[9];
UINT8 p_rom[8];

/***********************************
 *   1-Wire Interface *
 ***********************************/
void OWIinit(void)
{
   DDRx_OWI_PORT  |=  _BV(OWI_LINE);          /*  1-Wire   */
   PORTx_OWI_PORT |=  _BV(OWI_LINE);          /*   1-Wire  1 */
}

/*****************************
 *     1  OWI     *
 *****************************/
void WriteBit1(void)
{
   cli();
   DDRx_OWI_PORT  |=  _BV(OWI_LINE);          /*  1-Wire   */

   PORTx_OWI_PORT &= ~_BV(OWI_LINE);          /*   1-Wire  0 */
   Wait_us(OWI_DELAY_A); nop();               //  nop()      2 
   PORTx_OWI_PORT |= _BV(OWI_LINE);           /*   1-Wire  1 */
   Wait_us(OWI_DELAY_B);
   sei();
}

/*****************************
 *     0  OWI     *
 *****************************/
void WriteBit0(void)
{
   cli();
   DDRx_OWI_PORT  |=  _BV(OWI_LINE);          /*  1-Wire   */

   PORTx_OWI_PORT &= ~_BV(OWI_LINE);          /*   1-Wire  0 */
   Wait_us(OWI_DELAY_C);
   PORTx_OWI_PORT |= _BV(OWI_LINE);           /*   1-Wire  1 */
   Wait_us(OWI_DELAY_D);
   sei();
}

/*****************************
 *       OWI     *
 *****************************/
UINT8 ReadBit(void)
{
   UINT8 bit;
   cli();
   DDRx_OWI_PORT  |=  _BV(OWI_LINE);          /*  1-Wire   */

   PORTx_OWI_PORT &= ~_BV(OWI_LINE);          /*   1-Wire  0 */
   Wait_us(OWI_DELAY_A); nop();               //  nop()      2 
   PORTx_OWI_PORT |= _BV(OWI_LINE);           /*   1-Wire  1 */
   Wait_us(OWI_DELAY_E);
   DDRx_OWI_PORT  &=  ~_BV(OWI_LINE);         /*  1-Wire   */
   bit = PINx_OWI_PORT;
   Wait_us(OWI_DELAY_F);
   sei();
   bit &= _BV(OWI_LINE);

   if (bit)
      return 0x80;
   else
      return 0x00;
}

/*********************************
 *   OWI *
 *********************************/
bool DetectPresence(void)
{
   UINT8 bit;
   cli();
   DDRx_OWI_PORT  |=  _BV(OWI_LINE);          /*  1-Wire   */

   PORTx_OWI_PORT &= ~_BV(OWI_LINE);          /*   1-Wire  0 */
   Wait_us(OWI_DELAY_H);
   DDRx_OWI_PORT  &=  ~_BV(OWI_LINE);         /*  1-Wire   */
   PORTx_OWI_PORT |= _BV(OWI_LINE);           /*   1-Wire  1 */
   Wait_us(OWI_DELAY_I);
   bit = PINx_OWI_PORT;
   Wait_us(OWI_DELAY_J);
   sei();
   bit &= _BV(OWI_LINE);

   if (bit)
      return false;
   else
      return true;
}

/*********************************
 *        OWI   *
 *********************************/
void SendByte(UINT8 byte)
{
   UINT8 i;

   for (i = 0; i < 8; i ++)
   {
      if(byte & 0x01)
         WriteBit1();
      else
         WriteBit0();

      byte >>= 1;
   }
}

/*********************************
 *         OWI      *
 *********************************/
UINT8 ReceiveByte(void)
{
   UINT8 i;
   UINT8 byte = 0;

   for (i = 0; i < 8; i ++)
   {
      byte >>= 1;
      byte |= ReadBit();
   }
   return byte;
}

/****************************************
 * The READ ROM command can be used   *
 * on a bus with a single slave to read *
 * the 64 bit unique identifier.        *
 ****************************************/
void ReadRom(UINT8 * p_rom)
{
   UINT8 i;

   SendByte(OWI_CMD_READ_ROM);
   for (i = 0; i < 8; i++)
   {
      p_rom[i] = ReceiveByte();
   }
}

/*************************************************************
 * The MATCH ROM command is used to address individual     *
 * slave devices on the bus. After the MATCH ROM command,  *
 * the complete 64 bit identifier is transmitted on the bus. *
 * When this is done, only the device with exactly this      *
 * identifier is allowed to answer until the next reset      *
 * pulse is received.                                        *
 *************************************************************/
void MatchRom(UINT8 * p_rom)
{
   UINT8 i;
   SendByte(OWI_CMD_MATCH_ROM);
   for (i = 0; i < 8; i++)
   {
      SendByte(p_rom[i]);
   }
}

/**************************************************************
*                                      *
***************************************************************/
void ReadScratchpad(UINT8 * scratchpad)
{
   UINT8 i;
   SendByte(DS18B20_READ_SCRATCHPAD);
   for (i = 0; i < 9; i++)
   {
      scratchpad[i] = ReceiveByte();
   }
}

/**************************************************************
 *  CRC8                                                *
 * byte -  ,     CRC          *
 * crc  -   CRC.    crc = 0     *
 **************************************************************/
UINT8 ComputeCRC8(UINT8 byte, UINT8 crc)
{
   UINT8 i;

   for (i = 0; i < 8; i++)
   {
      if (((crc ^ byte) & 0x01))
      {
         crc ^= 0x18;
         crc >>= 1;
         crc |= 0x80;
      }
      else
      {
         crc >>= 1;
      }
      byte >>= 1;
   }
   return crc;
}


/******************************************************************
 *                 CRC8                *
 *  : CalcCRC(&rom[0], 7) -  CRC   *
 ******************************************************************/
UINT8 CalcCRC(UINT8 *p_array, UINT8 array_length)
{
   UINT8 i;
   UINT8 crc = 0;

   for (i = 0; i < array_length; i++)
   {
      crc = ComputeCRC8(p_array[i], crc);
   }
   
   if (crc == p_array[i])
      return 1;
   else
      return 0;
}

/**************************************************************
 *                        *
 **************************************************************/
INT16 ReadTemp(void)
{
   DetectPresence();
   MatchRom(p_rom);     // ,    
   SendByte(DS18B20_START_CONVERSION);
   Wait_ms(750);        //      750 
   DetectPresence();
   MatchRom(p_rom);     // ,    
   ReadScratchpad(&scratchpad[0]);
   return (scratchpad[0] + (scratchpad[1] << 8));
}

/******************************************************************
 *                DS18B20              *
 ******************************************************************/
bool GetTemp(char *temperature)
{
   INT16   current_temp;
   INT16   temp_base;
   bool    is_ok = false;

   if (DetectPresence())          //      
   {
      ReadRom(p_rom);             //    ROM (.. ID)
      current_temp = ReadTemp();  //  

      if (CalcCRC(&scratchpad[0], 8)) //    , ...
      {
         if (current_temp > T99_9_TEMPERATURE) current_temp = T99_9_TEMPERATURE;

         temp_base = current_temp;

         //   ,   
         if (temp_base < 0) temp_base = -temp_base;

         temp_base >>= 4;

         if ((current_temp < 0) && (temp_base >= 1)) *(temperature++) = 'M'; // 'M'      ""

         buff_print_dec(&temperature, (UINT8)temp_base);

         *(temperature++) = 'C'; // 'C'   

         *(temperature++) = '\0';

         is_ok = true;
      }
   }
   return is_ok;
}

